#include<stdio.h>
#include<stdlib.h>
#include<math.h>
#include<gsl/gsl_math.h>
#include<gsl/gsl_integration.h>

/*la funcion de densidad Rho el a distribución original entre una rho caractaristica de la misma distribución y es dependiente de R, que es la coordenada esférica r entre una longitud caractarística de la */
double Rho(double R)
{
	if(R<=1)return 1.0 / (R * R);
	else return 0.0;
}

/*aqui esta el calculo de la funcion potencial*/
double Potencial_1(double Ri, void *params)/*nombre el puntero params sin valor especifico que necesita la funcion*/
{
	double alpha = *(double *)params;
	return Ri * Ri * Rho(Ri);
}

double Potencial_2(double Ue, void *params)/*nombre el puntero params sin valor especifico que necesita la funcion*/
{
	double alpha = *(double *)params;
	return Rho(1.0/Ue) / (Ue*Ue*Ue);
}

/*esta parte se utiliza para hacer las integrales definidas que corresponden al potencial*/
double calcular_integral_evaluada(double lim_inf, double lim_sup, int parte)
{
	gsl_integration_workspace*w = gsl_integration_workspace_alloc(1000);/*basicamente define un espacio de trabajo en w, con 1000 intervalos de doble precision*/

	double result, error;
	double alpha = 1.0;
	
	gsl_function F;
	if( parte==1 )F.function = &Potencial_1;
	if( parte==2 )F.function = &Potencial_2;
	F.params = &alpha;

	gsl_integration_qags (&F, lim_inf, lim_sup, 0.0, 1e-7, 1000, w, &result, &error); /*int gsl_integration_qags(const gsl_function * f, double a, double b, double epsabs, double epsrel, size_t limit, gsl_integration_workspace * workspace, double * result, double * abserr), regresa la cantidad "result"*/

	gsl_integration_workspace_free (w);

	return result;
}

double Potencial(double U)
{
	double P0 = -0.1;
	
	/*el limire superior del primero devería ser R, pero lo quiero hacer todo dependiente de U*/
	/*printf("%f\n", P0 * ( U * calcular_integral_evaluada(0.0 , 1.0/U , 1) + calcular_integral_evaluada(0.0 , U ,2) ) )*/;
	return P0 * ( U * calcular_integral_evaluada(0.0 , 1.0/U , 1) + calcular_integral_evaluada(0.0 , U ,2) );
}

/*Defino g00 y gRR*/
double g00(double U)
{
	return 1.0 + 2.0 * Potencial(U);
}

double gRR(double U)
{
	double gamma = 1.0;
	return -1.0 + 2.0 * gamma * Potencial(U);
}

/*Defino la función dPhi/dR encontrada para el ángulo de deflección con métrica simétricamente esférica*/
double dPhidR(double U, double y, double U0)
{
	return sqrt( ( 1.0 * g00(U) * gRR(U) ) / ((U*U*g00(U)) - (U0*U0*g00(U0))) );
}

/*El método utilizado para integrar será el de Simpson Compuesto*/
void integral_definida()
{
	double limite_inferior, limite_superior, U, Phi, U0, k1, k2, k3, k4, dU;

	U0= 0.9;
	limite_inferior=0.0000000001;
	limite_superior=U0;
	dU=0.00001;

	/*aqui definimos nuestro punto inicial para la integración, el punto U y su valor respectivo de Phi*/
	U = limite_inferior;
	Phi = M_PI;
	
	do
	{
		k1 = dPhidR( U , Phi , U0 );
		k2 = dPhidR( U+0.5*dU , Phi+0.5*dU*k1 , U0 );
		k3 = dPhidR( U+0.5*dU , Phi+0.5*dU*k2 , U0 );
		k4 = dPhidR( U+dU , Phi+k3*dU , U0 );
		Phi = Phi + ( k1 + 2.0*k2 + 2.0*k3 + k4 ) * dU/6.0;
		U = U + dU;
		if( U < limite_superior )printf("%f\t%f\n",-Phi + 2.0*M_PI, 1.0/U);
	}while( U < limite_superior );
	return ;
}

/*Este es el cuerpo principal del programa*/
int main()
{
	integral_definida();
	return 0;
}
